# Chapter 5 Code for Ward & Gleditsch, Spatial Regression Models
# Sage, 2018.

# R version 3.4.3 (2017-11-30) -- "Kite-Eating Tree"
# Copyright (C) 2017 The R Foundation for Statistical Computing
# Platform: x86_64-apple-darwin15.6.0 (64-bit)

rm(list = ls())
library(cshapes) 
library(wbstats)
p5<-read.csv("rawdata/p5.csv")
wmap <- cshp(date = as.Date("2015-01-01"),useGW=TRUE)  # get map from CSHAPES
wmap$ISO1AL3 <- as.character(wmap$ISO1AL3)
wmap$ISO1AL3[wmap$CNTRY_NAME == "Kosovo"] <- c("RKS")
wmap$ISO1AL3 <- as.factor(wmap$ISO1AL3)
wmap@data$id <- as.character(rownames(wmap@data))
wmap.df <- wmap@data
wmap@data$democ <- p5$democ[match(wmap@data$ISO1AL3, p5$scode)]
wmap@data$autoc <- p5$autoc[match(wmap@data$ISO1AL3, p5$scode)]
wmap@data$polity <- wmap@data$democ - wmap@data$autoc

# now get GDP per capita data
library(wbstats)
gdp <- wb(indicator = c("NY.GDP.PCAP.CD"),
          startdate = 2014, enddate = 2014,country="countries_only")
gdp$value <- log(gdp$value)
# join the GDP data to the map data.frame
wmap@data$gdp <- gdp$value[match(wmap.df$ISO1AL2, gdp$iso2c)]

# make wmap@data a small data matrix
a.df<-subset.data.frame(wmap@data,select=c("polity","gdp"))
dim(a.df)
# now impute missing data in gdp and polity with sbgcop
library(sbgcop)
new.df<-sbgcop.mcmc(a.df,seed=123456,nsamp=10000)
non.miss.df<-data.frame(new.df$Y.pmean)
row.names(non.miss.df)<-wmap@data$GWCODE
non.miss.df$country<-wmap@data$CNTRY_NAME

# now get distance matrix
wmap.dist <- distmatrix(date = as.Date("2015-01-01")) 

# recode distance data to be binary, for distance < 400km
w.dist<-wmap.dist
w.dist[w.dist<=400]<-1
w.dist[w.dist>400]<-0
# row standardize
w.dist.rstd<-w.dist/rowSums(w.dist)
# 0 on diagonal
diag(w.dist.rstd)<-0

# create spatial lag of polity. round result
non.miss.df$sp.lag.polity<- round((w.dist.rstd)%*%(non.miss.df$polity))
# order map dataframe on GWCODE
nm.df<-non.miss.df[order(wmap@data$GWCODE),]

# get rid of isolates
omits<-(which(rowSums(w.dist.rstd)==0))

# get rid of isolates
nm.df<-nm.df[-omits,]
w.d.rstd<-w.dist.rstd[-omits,-omits]
dim(w.d.rstd)
dim(nm.df)

# spatial model
library(spdep)
listw.obj<-mat2listw(w.d.rstd)
# spatial lag
sar.fit<-lagsarlm(polity ~ gdp, data=nm.df, listw.obj,type="lag",
                  method="eigen")
# spatial error
sem.fit <- errorsarlm(polity ~ gdp, data=nm.df,
                      listw.obj, method="eigen", quiet=FALSE)
# OLS
ols.fit <- glm(polity ~ gdp, data=nm.df)

stargazer(ols.fit,sar.fit,sem.fit)

# SLIGHTLY DIFFERENT THAN 5.1 in Book owing to using imputed data.

