# Chapter 5 Code for Ward & Gleditsch, Spatial Regression Models
# Sage, 2018.

# R version 3.4.3 (2017-11-30) -- "Kite-Eating Tree"
# Copyright (C) 2017 The R Foundation for Statistical Computing
# Platform: x86_64-apple-darwin15.6.0 (64-bit)

# Europe trade example

# Matrix with reverse dyads
cmat98BA <- read.table("rawdata/tradedata/dyadmat_europe98_BA-no origins 3+.csv",sep=",",header=T)

# Matrix without  with reverse dyads
cmat98noBA <- read.table("rawdata/tradedata/dyadmat_europe98_no BA-no origins 3+.csv",sep=",",header=T)

# Data
logdat98 <- read.table("rawdata/tradedata//dyaddata_europe98_log-no origins 3+.csv",sep=",",header=T)

# Make distance matrix a list
dlist <- vector(mode="list",length=dim(logdat98)[1])
attr(dlist,"region.id") <- logdat98$id
attr(dlist,"class") <- "nb"
ids <- unique(cmat98BA$c1)
for(i in 1:length(ids)){
  # Find their numbers in the data
  dat1 <- cmat98BA[cmat98BA$c1==ids[i],]
  dat2 <- cmat98noBA[cmat98noBA$c1==ids[i],]
  input <- sort(c(dat1$c2,dat2$c2))
  dlist[[i]] <- input
}

# OLS
tab2.ols <- glm(logtrade ~ logdem + logapop + logbpop + logargdppc + logbrgdppc + logs + logdist + logmid,
                data=logdat98,na.action=na.omit)
logLik(tab2.ols)

# SEM
tab2.sem <- errorsarlm(logtrade ~ logdem + logapop + logbpop + logargdppc + logbrgdppc + logs + logdist + logmid,
                       data=logdat98,na.action=na.omit,
                       nb2listw(dlist,style="W"), method="eigen", quiet=FALSE)
summary(tab2.sem)
logLik(tab2.sem)

tab<-stargazer(tab2.ols, tab2.sem,type = "latex",star.char = NULL,nobs=TRUE,digits=2,
          star.cutoffs = NA,notes.append = FALSE, 
          notes        = "Exports, Europe:",dep.var.caption  = "Exports, Europe:"
)

print(tab)

